<?php

declare(strict_types=1);

namespace Drupal\Tests\visitors\Kernel\Plugin;

use Drupal\KernelTests\KernelTestBase;
use Drupal\visitors\Plugin\VisitorsActionBase;

/**
 * Tests the VisitorsActionBase plugin base class.
 *
 * @group visitors
 * @coversDefaultClass \Drupal\visitors\Plugin\VisitorsActionBase
 */
class VisitorsActionBaseTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'visitors',
  ];

  /**
   * The visitors action plugin manager.
   *
   * @var \Drupal\visitors\VisitorsActionPluginManager
   */
  protected $actionPluginManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('user');
    $this->installSchema('system', ['sequences']);
    $this->installConfig(['user']);

    $this->actionPluginManager = $this->container->get('visitors.action_plugin_manager');
  }

  /**
   * Creates a test plugin instance.
   *
   * @param array $pluginDefinition
   *   The plugin definition.
   *
   * @return \Drupal\visitors\Plugin\VisitorsActionBase
   *   The test plugin instance.
   */
  protected function createTestPlugin(array $pluginDefinition): VisitorsActionBase {
    return new class([], 'test_action', $pluginDefinition) extends VisitorsActionBase {

      /**
       * {@inheritdoc}
       */
      public function process(array $context): ?array {
        return NULL;
      }

    };
  }

  /**
   * Tests the plugin creation and basic functionality.
   *
   * @covers ::getLabel
   * @covers ::getPlugin
   * @covers ::getWeight
   */
  public function testPluginCreation(): void {
    // Create a mock plugin definition.
    $plugin_definition = [
      'id' => 'test_action',
      'label' => 'Test Action',
      'plugin' => 'test_plugin',
      'weight' => 10,
    ];

    // Create a mock plugin instance.
    $plugin = $this->createTestPlugin($plugin_definition);

    // Test basic functionality.
    $this->assertEquals($plugin_definition['label'], $plugin->getLabel());
    $this->assertEquals($plugin_definition['plugin'], $plugin->getPlugin());
    $this->assertEquals($plugin_definition['weight'], $plugin->getWeight());
  }

  /**
   * Tests the plugin manager integration.
   */
  public function testPluginManagerIntegration(): void {
    // Get all plugin definitions.
    $definitions = $this->actionPluginManager->getDefinitions();

    // Ensure we have plugin definitions.
    $this->assertNotEmpty($definitions);

    // Test each plugin definition.
    foreach ($definitions as $definition) {
      $plugin = $this->actionPluginManager->createInstance($definition['id']);

      // Verify the plugin is an instance of VisitorsActionBase.
      $this->assertInstanceOf(VisitorsActionBase::class, $plugin);

      // Test basic functionality.
      $this->assertEquals($definition['label'], $plugin->getLabel());
      $this->assertEquals($definition['plugin'] ?? get_class($plugin), $plugin->getPlugin());
      $this->assertEquals($definition['weight'] ?? 0, $plugin->getWeight());
    }
  }

}
